@@ -2937,9 +2937,9 @@ int filledEllipseRGBA(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rx, Sint16 r
 
 /* ----- filled pie */
 
-/* Low-speed float pie-calc implementation by drawing polygons. */
+/* Low-speed float pie-calc implementation by drawing polygons/lines. */
 
-int filledpieColor(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rad, Sint16 start, Sint16 end, Uint32 color)
+int doPieColor(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rad, Sint16 start, Sint16 end, Uint32 color, Uint8 filled)
 {
     Sint16 left, right, top, bottom;
     Sint16 x1, y1, x2, y2;
@@ -3049,7 +3049,11 @@ int filledpieColor(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rad, Sint16 sta
     }
 
     /* Draw */
-    result = filledPolygonColor(dst, vx, vy, numpoints, color);
+    if (filled) {
+     result = filledPolygonColor(dst, vx, vy, numpoints, color);
+    } else {
+     result = polygonColor(dst, vx, vy, numpoints, color);
+    }
 
     /* Free vertex array */
     free(vx);
@@ -3057,14 +3061,32 @@ int filledpieColor(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rad, Sint16 sta
     return (result);
 }
 
-int filledpieRGBA(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rad,
-		  Sint16 start, Sint16 end, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
+int pieColor(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rad, 
+		Sint16 start, Sint16 end, Uint32 color) 
 {
-    return (filledpieColor(dst, x, y, rad, start, end,
-			   ((Uint32) r << 24) | ((Uint32) g << 16) | ((Uint32) b << 8) | (Uint32) a));
+    return (doPieColor(dst, x, y, rad, start, end, color, 0));
 
 }
 
+int pieRGBA(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rad,
+	    Sint16 start, Sint16 end, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
+{
+    return (doPieColor(dst, x, y, rad, start, end,
+			   ((Uint32) r << 24) | ((Uint32) g << 16) | ((Uint32) b << 8) | (Uint32) a, 0));
+
+}
+
+int filledPieColor(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rad, Sint16 start, Sint16 end, Uint32 color)
+{
+    return (doPieColor(dst, x, y, rad, start, end, color, 1));
+}
+
+int filledPieRGBA(SDL_Surface * dst, Sint16 x, Sint16 y, Sint16 rad,
+		  Sint16 start, Sint16 end, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
+{
+    return (doPieColor(dst, x, y, rad, start, end,
+			   ((Uint32) r << 24) | ((Uint32) g << 16) | ((Uint32) b << 8) | (Uint32) a, 1));
+}
 
 /* Trigon */
 
@@ -3134,7 +3156,7 @@ int aatrigonRGBA(SDL_Surface * dst,  Sint16 x1, Sint16 y1, Sint16 x2, Sint16 y2,
 
 /* Filled Trigon */
 
-int filledTrigonColor(SDL_Surface * dst, Sint16 x1, Sint16 y1, Sint16 x2, Sint16 y2, Sint16 x3, Sint16 y3, int color)
+int filledTrigonColor(SDL_Surface * dst, Sint16 x1, Sint16 y1, Sint16 x2, Sint16 y2, Sint16 x3, Sint16 y3, Uint32 color)
 {
  Sint16 vx[3]; 
  Sint16 vy[3];
@@ -3167,11 +3189,11 @@ int filledTrigonRGBA(SDL_Surface * dst, Sint16 x1, Sint16 y1, Sint16 x2, Sint16
 
 /* ---- Polygon */
 
-int polygonColor(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, Uint32 color)
+int polygonColor(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, int n, Uint32 color)
 {
     int result;
     int i;
-    Sint16 *x1, *y1, *x2, *y2;
+    const Sint16 *x1, *y1, *x2, *y2;
 
     /*
      * Sanity check 
@@ -3204,7 +3226,7 @@ int polygonColor(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, Uint32 colo
     return (result);
 }
 
-int polygonRGBA(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
+int polygonRGBA(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, int n, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
 {
     /*
      * Draw 
@@ -3214,11 +3236,11 @@ int polygonRGBA(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, Uint8 r, Uin
 
 /* ---- AA-Polygon */
 
-int aapolygonColor(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, Uint32 color)
+int aapolygonColor(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, int n, Uint32 color)
 {
     int result;
     int i;
-    Sint16 *x1, *y1, *x2, *y2;
+    const Sint16 *x1, *y1, *x2, *y2;
 
     /*
      * Sanity check 
@@ -3251,7 +3273,7 @@ int aapolygonColor(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, Uint32 co
     return (result);
 }
 
-int aapolygonRGBA(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
+int aapolygonRGBA(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, int n, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
 {
     /*
      * Draw 
@@ -3266,7 +3288,7 @@ int gfxPrimitivesCompareInt(const void *a, const void *b);
 static int *gfxPrimitivesPolyInts = NULL;
 static int gfxPrimitivesPolyAllocated = 0;
 
-int filledPolygonColor(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, int color)
+int filledPolygonColor(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, int n, Uint32 color)
 {
     int result;
     int i;
@@ -3338,7 +3360,7 @@ int filledPolygonColor(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, int c
 		continue;
 	    }
 	    if ( ((y >= y1) && (y < y2)) || ((y == maxy) && (y > y1) && (y <= y2)) ) {
-		gfxPrimitivesPolyInts[ints++] = (65536 * (y - y1) * (x2 - x1)) / (y2 - y1) + (65536 * x1);
+		gfxPrimitivesPolyInts[ints++] = ((65536 * (y - y1)) / (y2 - y1)) * (x2 - x1) + (65536 * x1);
 	    } 
 	    
 	}
@@ -3357,7 +3379,7 @@ int filledPolygonColor(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, int c
     return (result);
 }
 
-int filledPolygonRGBA(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
+int filledPolygonRGBA(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, int n, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
 {
     /*
      * Draw 
@@ -3371,11 +3393,43 @@ int gfxPrimitivesCompareInt(const void *a, const void *b)
     return (*(const int *) a) - (*(const int *) b);
 }
 
-/* ---- Character (8x8 internal font) */
+/* ---- Character */
 
 static SDL_Surface *gfxPrimitivesFont[256];
 static Uint32 gfxPrimitivesFontColor[256];
 
+/* Default is to use 8x8 internal font */
+static const unsigned char *currentFontdata = gfxPrimitivesFontdata;
+
+static int charWidth = 8, charHeight = 8;
+static int charPitch = 1;
+static int charSize = 8;	/* character data size in bytes */
+
+void gfxPrimitivesSetFont(const void *fontdata, int cw, int ch)
+{
+    int i;
+
+    if (fontdata) {
+        currentFontdata = fontdata;
+        charWidth = cw;
+        charHeight = ch;
+    } else {
+        currentFontdata = gfxPrimitivesFontdata;
+        charWidth = 8;
+        charHeight = 8;
+    }
+
+    charPitch = (charWidth+7)/8;
+    charSize = charPitch * charHeight;
+
+    for (i = 0; i < 256; i++) {
+	if (gfxPrimitivesFont[i]) {
+	    SDL_FreeSurface(gfxPrimitivesFont[i]);
+	    gfxPrimitivesFont[i] = NULL;
+	}
+    }
+}
+
 int characterColor(SDL_Surface * dst, Sint16 x, Sint16 y, char c, Uint32 color)
 {
     Sint16 left, right, top, bottom;
@@ -3383,12 +3437,12 @@ int characterColor(SDL_Surface * dst, Sint16 x, Sint16 y, char c, Uint32 color)
     SDL_Rect srect;
     SDL_Rect drect;
     int result;
-    int ix, iy, k;
-    unsigned char *charpos;
-    unsigned char bits[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
+    int ix, iy;
+    const unsigned char *charpos;
     unsigned char *bitpos;
     Uint8 *curpos;
     int forced_redraw;
+    Uint8 patt, mask;
 
     /*
      * Get clipping boundary 
@@ -3402,9 +3456,9 @@ int characterColor(SDL_Surface * dst, Sint16 x, Sint16 y, char c, Uint32 color)
      * Test if bounding box of character is visible 
      */
     x1 = x;
-    x2 = x + 8;
+    x2 = x + charWidth;
     y1 = y;
-    y2 = y + 8;
+    y2 = y + charHeight;
     if ((x1<left) && (x2<left)) {
      return(0);
     } 
@@ -3419,28 +3473,29 @@ int characterColor(SDL_Surface * dst, Sint16 x, Sint16 y, char c, Uint32 color)
     } 
 
     /*
-     * Setup source rectangle for 8x8 bitmap 
+     * Setup source rectangle
      */
     srect.x = 0;
     srect.y = 0;
-    srect.w = 8;
-    srect.h = 8;
+    srect.w = charWidth;
+    srect.h = charHeight;
 
     /*
-     * Setup destination rectangle for 8x8 bitmap 
+     * Setup destination rectangle
      */
     drect.x = x;
     drect.y = y;
-    drect.w = 8;
-    drect.h = 8;
+    drect.w = charWidth;
+    drect.h = charHeight;
 
     /*
-     * Create new 8x8 bitmap surface if not already present 
+     * Create new charWidth x charHeight bitmap surface if not already present 
      */
     if (gfxPrimitivesFont[(unsigned char) c] == NULL) {
 	gfxPrimitivesFont[(unsigned char) c] =
-	    SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_HWSURFACE | SDL_SRCALPHA, 8, 8,
-				 32, 0xFF000000, 0x00FF0000, 0x0000FF00, 0x000000FF);
+	    SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_HWSURFACE | SDL_SRCALPHA,
+                                 charWidth, charHeight, 32,
+				 0xFF000000, 0x00FF0000, 0x0000FF00, 0x000000FF);
 	/*
 	 * Check pointer 
 	 */
@@ -3468,30 +3523,27 @@ int characterColor(SDL_Surface * dst, Sint16 x, Sint16 y, char c, Uint32 color)
 	/*
 	 * Variable setup 
 	 */
-	k = (unsigned char) c;
-	k *= 8;
-	charpos = gfxPrimitivesFontdata;
-	charpos += k;
-
-	/*
-	 * Clear bitmap 
-	 */
+	charpos = currentFontdata + (unsigned char) c * charSize;
 	curpos = (Uint8 *) gfxPrimitivesFont[(unsigned char) c]->pixels;
-	memset(curpos, 0, 8 * 8 * 4);
 
 	/*
 	 * Drawing loop 
 	 */
-	for (iy = 0; iy < 8; iy++) {
-	    bitpos = bits;
-	    for (ix = 0; ix < 8; ix++) {
-		if ((*charpos & *bitpos) == *bitpos) {
-		    memcpy(curpos, &color, 4);
+        patt = 0;
+	for (iy = 0; iy < charHeight; iy++) {
+            mask = 0x00;
+	    for (ix = 0; ix < charWidth; ix++) {
+		if (!(mask >>= 1)) {
+		    patt = *charpos++;
+		    mask = 0x80;
 		}
-		bitpos++;
+
+		if (patt & mask)
+		    *(Uint32 *)curpos = color;
+		else
+		    *(Uint32 *)curpos = 0;
 		curpos += 4;;
 	    }
-	    charpos++;
 	}
     }
 
@@ -3511,27 +3563,22 @@ int characterRGBA(SDL_Surface * dst, Sint16 x, Sint16 y, char c, Uint8 r, Uint8
     return (characterColor(dst, x, y, c, ((Uint32) r << 24) | ((Uint32) g << 16) | ((Uint32) b << 8) | (Uint32) a));
 }
 
-int stringColor(SDL_Surface * dst, Sint16 x, Sint16 y, char *c, Uint32 color)
+int stringColor(SDL_Surface * dst, Sint16 x, Sint16 y, const char *c, Uint32 color)
 {
-    int result;
-    int i, length;
-    char *curchar;
-    int curx;
-
-    length = strlen(c);
-    curchar = c;
-    curx = x;
-    result = 0;
-    for (i = 0; i < length; i++) {
+    int result = 0;
+    int curx = x;
+    const char *curchar = c;
+ 
+    while (*curchar) {
 	result |= characterColor(dst, curx, y, *curchar, color);
+	curx += charWidth;
 	curchar++;
-	curx += 8;
     }
 
     return (result);
 }
 
-int stringRGBA(SDL_Surface * dst, Sint16 x, Sint16 y, char *c, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
+int stringRGBA(SDL_Surface * dst, Sint16 x, Sint16 y, const char *c, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
 {
     /*
      * Draw 
@@ -3592,7 +3639,7 @@ double evaluateBezier (double *data, int ndata, double t)
  return(result);
 }
 
-int bezierColor(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, int s, Uint32 color)
+int bezierColor(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, int n, int s, Uint32 color)
 {
     int result;
     int i;
@@ -3652,7 +3699,7 @@ int bezierColor(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, int s, Uint3
     return (result);
 }
 
-int bezierRGBA(SDL_Surface * dst, Sint16 * vx, Sint16 * vy, int n, int s, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
+int bezierRGBA(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, int n, int s, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
 {
     /*
      * Draw 
