@@ -12,7 +12,7 @@
 #include <math.h>
 #include <time.h>
 
-#include <SDL/SDL.h>
+#include "SDL.h"
 
 #include "SDL_gfxPrimitives.h"
 
@@ -457,6 +457,87 @@ void BenchmarkRectangle(SDL_Surface *screen)
 
 }
 
+void TestBoxTwo(SDL_Surface *screen)
+{
+ int i;
+ char r,g,b;
+ 
+ /* Create random points */
+ srand(time(NULL));
+ InitRandomPoints();
+
+ /* Draw A=255 */
+ SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
+ for (i=0; i<NUM_RANDOM; i++) {
+  boxRGBA(screen, rx1[i], ry1[i], rx1[i]+1, ry1[i]+1, rr[i], rg[i], rb[i], 255);
+ }
+
+ /* Draw A=various */
+ SetClip(screen,WIDTH/2,60,WIDTH,60+(HEIGHT-80)/2);
+ for (i=0; i<NUM_RANDOM; i++) {
+  boxRGBA(screen, rx2[i], ry1[i], rx2[i]+1, ry1[i]+1, rr[i], rg[i], rb[i], ra[i]);
+ }
+
+ /* Draw A=various */
+ SetClip(screen,WIDTH/2,80+(HEIGHT-80)/2,WIDTH,HEIGHT);
+ for (i=0; i<NUM_RANDOM; i++) {
+  boxRGBA(screen, rx2[i], ry2[i], rx2[i]+1, ry2[i]+1, rr[i], rg[i], rb[i], ra[i]);
+ }
+
+ /* Draw Colortest */
+ SetClip(screen,0,80+(HEIGHT-80)/2,WIDTH/2,HEIGHT);
+ for (i=0; i<NUM_RANDOM; i++) {
+  if (rx1[i] < (WIDTH/6))  {
+   r=255; g=0; b=0; 
+  } else if (rx1[i] < (WIDTH/3) ) {
+   r=0; g=255; b=0; 
+  } else {
+   r=0; g=0; b=255; 
+  }
+  boxRGBA(screen, rx1[i], ry2[i], rx1[i]+1, ry2[i]+1, r, g, b, 255);
+ }
+}
+
+void BenchmarkBoxTwo(SDL_Surface *screen)
+{
+ int i,j;
+ int repeat;
+ Uint32 time1, time2;
+ char titletext[256];
+ 
+ /* Draw A=255 */
+ repeat=50;
+ SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
+ time1=SDL_GetTicks();
+ for (j=0; j<repeat; j++) {
+  for (i=0; i<NUM_RANDOM; i++) {
+   boxRGBA(screen, rx1[i], ry1[i], rx1[i]+1, ry1[i]+1, rr[i], rg[i], rb[i], 255);
+  }
+ }
+ time2=SDL_GetTicks();
+ /* Results */
+ SetClip(screen,0,0,WIDTH-1,HEIGHT-1);
+ sprintf (titletext, "%.0f per sec",1000.0*(float)(NUM_RANDOM*repeat)/(float)(time2-time1));
+ stringRGBA (screen, WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
+ 
+ /* Draw A=various */
+ repeat=50;
+ SetClip(screen,WIDTH/2,60,WIDTH,60+(HEIGHT-80)/2);
+ time1=SDL_GetTicks();
+ for (j=0; j<repeat; j++) {
+  for (i=0; i<NUM_RANDOM; i++) {
+   boxRGBA(screen, rx2[i], ry1[i], rx2[i]+1, ry1[i]+1, rr[i], rg[i], rb[i], ra[i]);
+  }
+ }
+ time2=SDL_GetTicks();
+
+ /* Results */
+ SetClip(screen,0,0,WIDTH-1,HEIGHT-1);
+ sprintf (titletext, "%.0f per sec",1000.0*(float)(NUM_RANDOM*repeat)/(float)(time2-time1));
+ stringRGBA (screen, 3*WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
+
+}
+
 void TestBox(SDL_Surface *screen)
 {
  int i;
@@ -1181,7 +1262,7 @@ void BenchmarkFilledEllipse(SDL_Surface *screen)
  stringRGBA (screen, 3*WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
 }
 
-void TestFilledpie(SDL_Surface *screen)
+void TestPie(SDL_Surface *screen)
 {
  int i;
  char r,g,b;
@@ -1193,19 +1274,19 @@ void TestFilledpie(SDL_Surface *screen)
  /* Draw A=255 */
  SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
  for (i=0; i<NUM_RANDOM; i++) {
-  filledpieRGBA(screen, rx1[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], 255);
+  pieRGBA(screen, rx1[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], 255);
  }
 
  /* Draw A=various */
  SetClip(screen,WIDTH/2,60,WIDTH,60+(HEIGHT-80)/2);
  for (i=0; i<NUM_RANDOM; i++) {
-  filledpieRGBA(screen, rx2[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
+  pieRGBA(screen, rx2[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
  }
 
  /* Draw A=various */
  SetClip(screen,WIDTH/2,80+(HEIGHT-80)/2,WIDTH,HEIGHT);
  for (i=0; i<NUM_RANDOM; i++) {
-  filledpieRGBA(screen, rx2[i], ry2[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
+  pieRGBA(screen, rx2[i], ry2[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
  }
 
  /* Draw Colortest */
@@ -1218,11 +1299,11 @@ void TestFilledpie(SDL_Surface *screen)
   } else {
    r=0; g=0; b=255; 
   }
-  filledpieRGBA(screen, rx1[i], ry2[i], rr1[i], a1[i], a2[i], r, g, b, 255);
+  pieRGBA(screen, rx1[i], ry2[i], rr1[i], a1[i], a2[i], r, g, b, 255);
  }
 }
 
-void BenchmarkFilledpie(SDL_Surface *screen)
+void BenchmarkPie(SDL_Surface *screen)
 {
  int i,j;
  int repeat;
@@ -1235,7 +1316,7 @@ void BenchmarkFilledpie(SDL_Surface *screen)
  time1=SDL_GetTicks();
  for (j=0; j<repeat; j++) {
   for (i=0; i<NUM_RANDOM; i++) {
-   filledpieRGBA(screen, rx1[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], 255);
+   pieRGBA(screen, rx1[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], 255);
   }
  }
  time2=SDL_GetTicks();
@@ -1250,7 +1331,87 @@ void BenchmarkFilledpie(SDL_Surface *screen)
  time1=SDL_GetTicks();
  for (j=0; j<repeat; j++) {
   for (i=0; i<NUM_RANDOM; i++) {
-   filledpieRGBA(screen, rx2[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
+   pieRGBA(screen, rx2[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
+  }
+ }
+ time2=SDL_GetTicks();
+
+ /* Results */
+ SetClip(screen,0,0,WIDTH-1,HEIGHT-1);
+ sprintf (titletext, "%.0f per sec",1000.0*(float)(NUM_RANDOM*repeat)/(float)(time2-time1));
+ stringRGBA (screen, 3*WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
+}
+
+void TestFilledPie(SDL_Surface *screen)
+{
+ int i;
+ char r,g,b;
+ 
+ /* Create random points */
+ srand(time(NULL));
+ InitRandomPoints();
+ 
+ /* Draw A=255 */
+ SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
+ for (i=0; i<NUM_RANDOM; i++) {
+  filledPieRGBA(screen, rx1[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], 255);
+ }
+
+ /* Draw A=various */
+ SetClip(screen,WIDTH/2,60,WIDTH,60+(HEIGHT-80)/2);
+ for (i=0; i<NUM_RANDOM; i++) {
+  filledPieRGBA(screen, rx2[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
+ }
+
+ /* Draw A=various */
+ SetClip(screen,WIDTH/2,80+(HEIGHT-80)/2,WIDTH,HEIGHT);
+ for (i=0; i<NUM_RANDOM; i++) {
+  filledPieRGBA(screen, rx2[i], ry2[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
+ }
+
+ /* Draw Colortest */
+ SetClip(screen,0,80+(HEIGHT-80)/2,WIDTH/2,HEIGHT);
+ for (i=0; i<NUM_RANDOM; i++) {
+  if (rx1[i] < (WIDTH/6))  {
+   r=255; g=0; b=0; 
+  } else if (rx1[i] < (WIDTH/3) ) {
+   r=0; g=255; b=0; 
+  } else {
+   r=0; g=0; b=255; 
+  }
+  filledPieRGBA(screen, rx1[i], ry2[i], rr1[i], a1[i], a2[i], r, g, b, 255);
+ }
+}
+
+void BenchmarkFilledPie(SDL_Surface *screen)
+{
+ int i,j;
+ int repeat;
+ Uint32 time1, time2;
+ char titletext[256];
+ 
+ /* Draw A=255 */
+ repeat=20;
+ SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
+ time1=SDL_GetTicks();
+ for (j=0; j<repeat; j++) {
+  for (i=0; i<NUM_RANDOM; i++) {
+   filledPieRGBA(screen, rx1[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], 255);
+  }
+ }
+ time2=SDL_GetTicks();
+ /* Results */
+ SetClip(screen,0,0,WIDTH-1,HEIGHT-1);
+ sprintf (titletext, "%.0f per sec",1000.0*(float)(NUM_RANDOM*repeat)/(float)(time2-time1));
+ stringRGBA (screen, WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
+ 
+ /* Draw A=various */
+ repeat=5;
+ SetClip(screen,WIDTH/2,60,WIDTH,60+(HEIGHT-80)/2);
+ time1=SDL_GetTicks();
+ for (j=0; j<repeat; j++) {
+  for (i=0; i<NUM_RANDOM; i++) {
+   filledPieRGBA(screen, rx2[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
   }
  }
  time2=SDL_GetTicks();
@@ -1583,7 +1744,7 @@ void BenchmarkBezier(SDL_Surface *screen)
  InitRandomPoints();
  
  /* Draw A=255 */
- repeat=50;
+ repeat=20;
  SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
  time1=SDL_GetTicks();
  for (j=0; j<repeat; j++) {
@@ -1735,9 +1896,18 @@ int main(int argc, char *argv[])
  			  /* Next primitive */ 			  
 	 		  oldprim=curprim; 
 			  break;
+
+			 /* --- 2x2 Box */
+			 case 6:
+		 	  ClearScreen(screen, "2x2 Box");
+			  TestBoxTwo(screen);
+			  BenchmarkBoxTwo(screen);
+ 			  /* Next primitive */ 			  
+	 		  oldprim=curprim; 
+			  break;
 			  
 			 /* --- Lines */
-			 case 6:
+			 case 7:
 		 	  ClearScreen(screen, "Line");
 			  TestLine(screen);
 			  BenchmarkLine(screen);
@@ -1746,7 +1916,7 @@ int main(int argc, char *argv[])
 			  break;
 
 			 /* --- AA Line */
-			 case 7:
+			 case 8:
 		 	  ClearScreen(screen, "Anti-Aliased Line");
 			  TestAALine(screen);
 			  BenchmarkAALine(screen);
@@ -1755,7 +1925,7 @@ int main(int argc, char *argv[])
 			  break;
 			  
 			 /* --- Circle */
-			 case 8:
+			 case 9:
 		 	  ClearScreen(screen, "Circle");
 			  TestCircle(screen);
 			  BenchmarkCircle(screen);			 
@@ -1764,7 +1934,7 @@ int main(int argc, char *argv[])
 			  break;
 
 			 /* --- AA Circle */
-			 case 9:
+			 case 10:
 		 	  ClearScreen(screen, "AA Circle");
 			  TestAACircle(screen);
 			  BenchmarkAACircle(screen);			 
@@ -1773,7 +1943,7 @@ int main(int argc, char *argv[])
 			  break;
 			  
 			 /* --- Filled Circle */
-			 case 10:
+			 case 11:
 		 	  ClearScreen(screen, "Filled Circle");
 			  TestFilledCircle(screen);
 			  BenchmarkFilledCircle(screen);
@@ -1782,7 +1952,7 @@ int main(int argc, char *argv[])
 			  break;
 
 			 /* --- Ellipse */
-			 case 11:
+			 case 12:
 		 	  ClearScreen(screen, "Ellipse");
 			  TestEllipse(screen);
 			  BenchmarkEllipse(screen);
@@ -1791,7 +1961,7 @@ int main(int argc, char *argv[])
 			  break;
 
 			 /* --- AA Ellipse */
-			 case 12:
+			 case 13:
 		 	  ClearScreen(screen, "AA Ellipse");
 			  TestAAEllipse(screen);
 			  BenchmarkAAEllipse(screen);
@@ -1800,7 +1970,7 @@ int main(int argc, char *argv[])
 			  break;
 
 			 /* --- Filled Ellipse */
-			 case 13:
+			 case 14:
 		 	  ClearScreen(screen, "Filled Ellipse");
 			  TestFilledEllipse(screen);
 			  BenchmarkFilledEllipse(screen);
@@ -1808,17 +1978,26 @@ int main(int argc, char *argv[])
 	 		  oldprim=curprim; 
 			  break;
 
+			 /* --- Pie */
+			 case 15:
+		 	  ClearScreen(screen, "Pie");
+			  TestPie(screen);
+			  BenchmarkPie(screen);
+ 			  /* Next primitive */ 			  
+	 		  oldprim=curprim; 
+			  break;
+
 			 /* --- Filled Pie */
-			 case 14:
+			 case 16:
 		 	  ClearScreen(screen, "Filled Pie");
-			  TestFilledpie(screen);
-			  BenchmarkFilledpie(screen);
+			  TestFilledPie(screen);
+			  BenchmarkFilledPie(screen);
  			  /* Next primitive */ 			  
 	 		  oldprim=curprim; 
 			  break;
 
 			 /* --- Polygon */
-			 case 15:
+			 case 17:
 		 	  ClearScreen(screen, "Polygon");
 			  TestPolygon(screen);
 			  BenchmarkPolygon(screen);
@@ -1827,7 +2006,7 @@ int main(int argc, char *argv[])
 			  break;
 			 
 			 /* --- AA-Polygon */
-			 case 16:
+			 case 18:
 		 	  ClearScreen(screen, "AA-Polygon");
 			  TestAAPolygon(screen);
 			  BenchmarkAAPolygon(screen);
@@ -1836,7 +2015,7 @@ int main(int argc, char *argv[])
 			  break;
 			 
 			 /* ---- Filled Polygon */ 
-			 case 17:
+			 case 19:
 		 	  ClearScreen(screen, "Filled Polygon");
 			  TestFilledPolygon(screen);
 			  BenchmarkFilledPolygon(screen);
@@ -1845,7 +2024,7 @@ int main(int argc, char *argv[])
 			  break;
 
 			 /* ---- Bezier Curve */ 
-			 case 18:
+			 case 20:
 		 	  ClearScreen(screen, "Bezier Curve");
 			  TestBezier(screen);
 			  BenchmarkBezier(screen);
